import string
import config

BIG = 0.9
MED = 0.75
SMALL = 0.5
TINY = 0.3

class NoSuchCountry(Exception):
    
    def __init__(self, value):
        self.value = value
        
    def __str__(self):
        return repr(self.value)
    
class NoSuchOcean(Exception):
    
    def __init__(self, value):
        self.value = value
        
    def __str__(self):
        return repr(self.value)    
    
class IntegrityProblem(Exception):
    
    def __init__(self, value):
        self.value = value
        
    def __str__(self):
        return repr(self.value)    
    
class Country(object):
    
    def __init__(self, name, size, color, borders, oceans):
        self.name = name
        self.size = size
        self.color = color
        self.borders = borders
        self.oceans = oceans
        self.price = 10 * (len(self.borders) + len(self.oceans))
        self.number = 1;    #Number of this country to put in a deck
        
        self.machine_name = string.lower(string.replace(name, ' ', '-')) 
        
    def __repr__(self):
        return str(self.name)    

class Ocean(object):
    
    def __init__(self, name, number, borders):
        self.name = name
        self.number = number    #Number of this ocean to put in a deck
        self.borders = borders
        
    def __repr__(self):
        return str(self.name)        

class Continent(object):
    
    def __init__(self):
        self.name = ''
        self.countries = []
        self.oceans = []
        pass

    def __repr__(self):
        return str(self.countries)      
        
    #Return the country with the given name
    def get_country(self, name):
        for country in self.countries:
            if country.name == name:
                return country
        #return False
        raise NoSuchCountry('No such country as ' + name + ' in ' + self.name)
    
    #Return the ocean with the given name
    def get_ocean(self, name):
        for ocean in self.oceans:
            if ocean.name == name:
                return ocean
        #return False
        raise NoSuchOcean('No such ocean as ' + name + ' in ' + self.name)    
    
    #Tests the country connections and ocean within this continent to make sure they match up
    def test_integrity(self):
        #Test country borders
        print '******************************'
        print 'Checking country borders...'
        print '******************************'
        for country in self.countries:
            for border_name in country.borders:
                bordering_country = self.get_country(border_name)
                for bordering_country_border_name in bordering_country.borders:
                    if bordering_country_border_name == country.name:  
                        match_found = True
                if match_found:
                    print 'Checked: ' + country.name + ' - ' + bordering_country.name
                    match_found = False
                    continue
                else:
                    raise IntegrityProblem(country.name + ' borders ' + bordering_country.name + ', but there is such border indicated in ' + bordering_country.name)
                
        #Test ocean borders, ocean-side
        print '******************************'
        print 'Checking oceans, ocean-side...'
        print '******************************'
        for ocean in self.oceans:
            for border_name in ocean.borders:
                bordering_country = self.get_country(border_name)
                for bordering_country_ocean_name in bordering_country.oceans:
                    if bordering_country_ocean_name == ocean.name:  
                        match_found = True 
                if match_found:
                    print 'Checked: ' + ocean.name + ' - ' + bordering_country.name
                    match_found = False
                    continue
                else:
                    raise IntegrityProblem(ocean.name + ' borders ' + bordering_country.name + ', but there is such ocean indicated in ' + bordering_country.name)

        #Test ocean borders, country-side
        print '******************************'
        print 'Checking oceans, country-side...'  
        print '******************************'      
        for country in self.countries:
            for ocean_name in country.oceans:
                bordering_ocean = self.get_ocean(ocean_name)
                for bordering_ocean_country_name in bordering_ocean.borders:
                    if bordering_ocean_country_name == country.name:  
                        match_found = True
                if match_found:
                    print 'Checked: ' + country.name + ' - ' + bordering_ocean.name
                    match_found = False
                    continue
                else:
                    raise IntegrityProblem(country.name + ' borders ' + bordering_ocean.name + ', but there is such border indicated in ' + bordering_ocean.name)                

        print '******************************'
        print 'Finished: everything is good!'
        print '******************************'
                                                                    
class Europe(Continent):
    
    def __init__(self):
        # Call the base class constructor
        Continent.__init__(self)     
           
        self.name = 'Europe'
        self.machine_name = string.lower(string.replace(self.name, ' ', '-')) 
        
        color = config.COLOR1;
        self.countries.append(Country("Austria", MED, color, ["Switzerland", "Germany", "Czech Republic", "Slovakia", "Hungary", "Slovenia", "Italy"], []))
        self.countries.append(Country("Spain", BIG, color, ["Portugal", "France"], ["Med", "Atlantic"]))
        self.countries.append(Country("Belgium", SMALL, color, ["France", "The Netherlands", "Luxemburg", "Germany"], ["Atlantic"]))
        self.countries.append(Country("Greece", BIG, color, ["Albania", "Macedonia", "Bulgaria", "Turkey"], ["Med"]))
        self.countries.append(Country("Romania", BIG, color, ["Bulgaria", "Serbia", "Hungary", "Ukraine", "Moldova", "Turkey"], []))        
        self.countries.append(Country("Belarus", BIG, color, ["Ukraine", "Poland", "Lithuania", "Latvia", "Russia"], []))
        self.countries.append(Country("Northern Ireland", SMALL, color, ["Ireland", "Scotland"], ["Atlantic"]))
        self.countries.append(Country("Finland", BIG, color, ["Russia", "Sweden", "Norway"], ["Baltic"]))
        self.countries.append(Country("Montenegro", SMALL, color, ["Bosnia & Herzegovina", "Serbia", "Albania"], ["Med"])) 
        
        color = config.COLOR2;
        self.countries.append(Country("France", BIG, color, ["England", "Spain", "Belgium", "Luxemburg", "Germany", "Switzerland", "Italy"], ["Med", "Atlantic"]))
        self.countries.append(Country("Sweden", BIG, color, ["Finland", "Norway", "Denmark"], ["Baltic", "Atlantic"]))
        self.countries.append(Country("Czech Republic", MED, color, ["Germany", "Poland", "Slovakia", "Austria"], []))
        self.countries.append(Country("Slovenia", SMALL, color, ["Italy", "Austria", "Hungary", "Croatia"], ["Med"]))
        self.countries.append(Country("Serbia", MED, color, ["Bosnia & Herzegovina", "Croatia", "Hungary", "Romania", "Bulgaria", "Macedonia", "Albania", "Montenegro"], []))
        self.countries.append(Country("Ukraine", BIG, color, ["Bulgaria", "Turkey", "Romania", "Moldova", "Hungary", "Slovakia", "Poland", "Belarus", "Russia"], []))
        self.countries.append(Country("Lithuania", MED, color, ["Poland", "Belarus", "Latvia"], ["Baltic"]))
        self.countries.append(Country("Scotland", MED, color, ["England", "Northern Ireland"], ["Atlantic"]))
        
        color = config.COLOR3;
        self.countries.append(Country("Portugal", MED, color, ["Spain"], ["Atlantic"]))
        self.countries.append(Country("Germany", BIG, color, ["Switzerland", "Luxemburg", "Belgium", "France", "The Netherlands", "Denmark", "Poland", "Czech Republic", "Austria"], ["Atlantic", "Baltic"]))
        self.countries.append(Country("England", MED, color, ["Wales", "Scotland", "France", "Ireland"], ["Atlantic"]))
        self.countries.append(Country("Hungary", MED, color, ["Croatia", "Slovenia", "Austria", "Slovakia", "Ukraine", "Romania", "Serbia"], []))
        self.countries.append(Country("Bosnia & Herzegovina", MED, color, ["Croatia", "Serbia", "Montenegro"], ["Med"]))
        self.countries.append(Country("Bulgaria", MED, color, ["Turkey", "Greece", "Macedonia", "Serbia", "Romania", "Ukraine"], []))
        self.countries.append(Country("Russia", BIG, color, ["Finland", "Estonia", "Latvia", "Belarus", "Ukraine", "Norway"], ["Baltic", "Atlantic"]))
        
        color = config.COLOR4;
        self.countries.append(Country("Turkey", BIG, color, ["Greece", "Bulgaria", "Romania", "Ukraine"], ["Med"]))  
        self.countries.append(Country("Macedonia", SMALL, color, ["Greece", "Albania", "Serbia", "Bulgaria"], []))
        self.countries.append(Country("Italy", BIG, color, ["France", "Switzerland", "Austria", "Slovenia"], ["Med"]))
        self.countries.append(Country("Luxemburg", TINY, color, ["France", "Belgium", "Germany"], []))
        self.countries.append(Country("Ireland", MED, color, ["Northern Ireland", "England"], ["Atlantic"]))
        self.countries.append(Country("Denmark", MED, color, ["Germany", "Sweden"], ["Baltic", "Atlantic"]))
        self.countries.append(Country("Estonia", SMALL, color, ["Russia", "Latvia"], ["Baltic"]))   
        self.countries.append(Country("Poland", BIG, color, ["Belarus", "Ukraine", "Slovakia", "Czech Republic", "Germany", "Lithuania"], ["Baltic"]))
        self.countries.append(Country("Moldova", SMALL, color, ["Ukraine", "Romania"], [])) 
        
        color = config.COLOR5;
        self.countries.append(Country("Switzerland", SMALL, color, ["Italy", "France", "Germany", "Austria"], []))  
        self.countries.append(Country("The Netherlands", MED, color, ["Belgium", "Germany"], ["Atlantic"]))
        self.countries.append(Country("Wales", SMALL, color, ["England"], ["Atlantic"]))
        self.countries.append(Country("Norway", BIG, color, ["Sweden", "Finland", "Russia"], ["Atlantic"]))
        self.countries.append(Country("Latvia", MED, color, ["Estonia", "Russia", "Lithuania", "Belarus"], ["Baltic"]))
        self.countries.append(Country("Slovakia", MED, color, ["Hungary", "Austria", "Czech Republic", "Poland", "Ukraine"], []))
        self.countries.append(Country("Albania", SMALL, color, ["Greece", "Macedonia", "Serbia", "Montenegro"], ["Med"]))   
        self.countries.append(Country("Croatia", MED, color, ["Bosnia & Herzegovina", "Serbia", "Hungary", "Slovenia"], ["Med"]))         
        
        #self.countries.append(Country("Test", color, ["Austria", "Spain", "Greece", "Romania"], ["Baltic"]))        
        
        #Oceans
        self.oceans.append(Ocean("Atlantic", 3, ["Sweden", "Russia", "Norway", "Denmark", "Germany", "The Netherlands", "Belgium", "France", "England", "Scotland", "Northern Ireland", "Ireland", "Wales", "Spain", "Portugal"]))
        self.oceans.append(Ocean("Baltic", 1, ["Sweden", "Finland", "Russia", "Estonia", "Latvia", "Lithuania", "Poland", "Germany", "Denmark"]))
        self.oceans.append(Ocean("Med", 2, ["Turkey", "Greece", "Albania", "Montenegro", "Bosnia & Herzegovina", "Croatia", "Slovenia", "Italy", "France", "Spain"]))
        
        #Colors
        self.colors = [config.COLOR1, config.COLOR2, config.COLOR3, config.COLOR4, config.COLOR5]
        
        #Size
        self.size = len(self.countries)
        
        #Map image
        self.map = "assets/images/map_europe.png"
        
        #Country images
        for country in self.countries:
            country.image = "assets/images/countries/" + self.machine_name + "/" + country.machine_name + ".png" 
        

class Africa(Continent):
    
    def __init__(self):
        # Call the base class constructor
        Continent.__init__(self)     
           
        self.name = 'Africa'
        self.machine_name = string.lower(string.replace(self.name, ' ', '-')) 
        
        color = config.COLOR1;
        self.countries.append(Country("Tunisia", SMALL, color, ["Libya", "Algeria"], ["Med"]))     
        self.countries.append(Country("Egypt", MED, color, ["Libya", "Sudan"], ["Med", "Indian"]))
        self.countries.append(Country("Niger", BIG, color, ["Algeria", "Mali", "Burkina Faso", "Benin", "Nigeria", "Chad", "Libya"], []))
        self.countries.append(Country("Senegal", SMALL, color, ["Mauritania", "Mali", "Guinea"], ["Atlantic"]))
        self.countries.append(Country("Gabon", SMALL, color, ["Cameroon", "Congo"], ["Atlantic"]))
        self.countries.append(Country("Uganda", SMALL, color, ["Sudan", "DROT Congo", "Rwanda", "Tanzania", "Kenya"], []))
        self.countries.append(Country("Zambia", MED, color, ["Angola", "Namibia", "DROT Congo", "Botswana", "Zimbabwe", "Mozambique", "Tanzania"], []))
        self.countries.append(Country("Madagascar", BIG, color, [], ["Indian"]))

        color = config.COLOR2;
        self.countries.append(Country("Libya", BIG, color, ["Tunisia", "Algeria", "Niger", "Chad", "Sudan", "Egypt"], ["Med"]))     
        self.countries.append(Country("Morocco", MED, color, ["Algeria", "Mauritania"], ["Med", "Atlantic"]))
        self.countries.append(Country("Guinea", SMALL, color, ["Senegal", "Sierra Leone", "Liberia", "Ivory Coast", "Mali",], ["Atlantic"]))
        self.countries.append(Country("Togo", SMALL, color, ["Ghana", "Burkina Faso", "Benin"], ["Atlantic"]))
        self.countries.append(Country("Cameroon", MED, color, ["Nigeria", "Chad", "Central African Republic", "Congo", "Gabon",], ["Atlantic"]))
        self.countries.append(Country("Somalia", MED, color, ["Ethiopia", "Kenya"], ["Indian"]))
        self.countries.append(Country("DROT Congo", BIG, color, ["Sudan", "Central African Republic", "Congo", "Angola", "Zambia", "Tanzania", "Rwanda", "Uganda",], ["Atlantic"]))
        self.countries.append(Country("Botswana", MED, color, ["Zambia", "Namibia", "South Africa", "Zimbabwe"], []))

        color = config.COLOR3;
        self.countries.append(Country("Algeria", BIG, color, ["Tunisia", "Libya", "Niger", "Mali", "Mauritania", "Morocco"], ["Med"]))     
        self.countries.append(Country("Ivory Coast", SMALL, color, ["Liberia", "Guinea", "Mali", "Burkina Faso", "Ghana"], ["Atlantic"]))
        self.countries.append(Country("Chad", BIG, color, ["Libya", "Niger", "Nigeria", "Cameroon", "Central African Republic", "Sudan"], []))
        self.countries.append(Country("Ethiopia", BIG, color, ["Eritrea", "Sudan", "Kenya", "Somalia"], ["Indian"]))
        self.countries.append(Country("Rwanda", SMALL, color, ["Tanzania", "DROT Congo", "Uganda"], []))
        self.countries.append(Country("Mozambique", BIG, color, ["Tanzania", "Zambia", "South Africa", "Zimbabwe"], ["Indian"]))
        self.countries.append(Country("Namibia", BIG, color, ["Angola", "Zambia", "Botswana", "South Africa"], ["Atlantic"]))
        self.countries.append(Country("Benin", BIG, color, ["Nigeria", "Niger", "Burkina Faso", "Togo"], ["Atlantic"]))
        
        color = config.COLOR4;
        self.countries.append(Country("Mauritania", BIG, color, ["Morocco", "Algeria", "Mali", "Senegal"], ["Atlantic"]))     
        self.countries.append(Country("Liberia", SMALL, color, ["Sierra Leone", "Guinea", "Ivory Coast"], ["Atlantic"]))
        self.countries.append(Country("Burkina Faso", SMALL, color, ["Mali", "Ivory Coast", "Ghana", "Togo", "Benin", "Niger"], []))
        self.countries.append(Country("Nigeria", MED, color, ["Benin", "Niger", "Chad", "Cameroon"], ["Atlantic"]))
        self.countries.append(Country("Sudan", BIG, color, ["Egypt", "Libya", "Chad", "Central African Republic", "Kenya", "DROT Congo", "Uganda", "Ethiopia", "Eritrea"], ["Indian"]))
        self.countries.append(Country("Congo", MED, color, ["Gabon", "Cameroon", "Central African Republic", "DROT Congo"], ["Atlantic"]))
        self.countries.append(Country("Tanzania", BIG, color, ["Kenya", "Uganda", "Rwanda", "DROT Congo", "Zambia", "Mozambique"], ["Indian"]))
        self.countries.append(Country("South Africa", BIG, color, ["Namibia", "Botswana", "Zimbabwe", "Mozambique"], ["Atlantic", "Indian"]))

        color = config.COLOR5;
        self.countries.append(Country("Mali", BIG, color, ["Algeria", "Mauritania", "Senegal", "Guinea", "Ivory Coast", "Burkina Faso", "Niger", "Algeria"], []))     
        self.countries.append(Country("Sierra Leone", SMALL, color, ["Guinea", "Liberia"], ["Atlantic"]))
        self.countries.append(Country("Ghana", SMALL, color, ["Ivory Coast", "Burkina Faso", "Togo"], ["Atlantic"]))
        self.countries.append(Country("Eritrea", SMALL, color, ["Sudan", "Ethiopia"], ["Indian"]))
        self.countries.append(Country("Central African Republic", MED, color, ["Chad", "Sudan", "DROT Congo", "Congo", "Cameroon"], []))
        self.countries.append(Country("Kenya", MED, color, ["Somalia", "Ethiopia", "Sudan", "Uganda", "Tanzania"], ["Indian"]))
        self.countries.append(Country("Angola", BIG, color, ["DROT Congo", "Zambia", "Namibia"], ["Atlantic"]))
        self.countries.append(Country("Zimbabwe", MED, color, ["Zambia", "Mozambique", "South Africa", "Botswana"], []))
        
        #Oceans
        self.oceans.append(Ocean("Atlantic", 3, ["Morocco", "Mauritania", "Senegal", "Guinea", "Sierra Leone", "Liberia", "Ivory Coast", "Ghana", "Togo", "Benin", "Nigeria", "Cameroon", "Gabon", "Congo", "DROT Congo", "Angola", "Namibia", "South Africa"]))
        self.oceans.append(Ocean("Indian", 1, ["South Africa", "Mozambique", "Madagascar", "Tanzania", "Kenya", "Somalia", "Ethiopia", "Eritrea", "Sudan", "Egypt"]))
        self.oceans.append(Ocean("Med", 2, ["Egypt", "Libya", "Tunisia", "Algeria", "Morocco"]))
        
        #Colors
        self.colors = [config.COLOR1, config.COLOR2, config.COLOR3, config.COLOR4, config.COLOR5]
        
        #Size
        self.size = len(self.countries)
        
        #Map image
        self.map = "assets/images/continents/africa.png"    
        self.map_text = "assets/images/continents/africa_text.png"   
        
        #Country images
        for country in self.countries:
            country.image = "assets/images/countries/" + self.machine_name + "/" + country.machine_name + ".png" 

# this runs the main function if this script is called to run.
#  If it is imported as a module, we don't run the main function.
if __name__ == "__main__":
    europe = Europe()
    africa = Africa()
    europe.test_integrity()
    africa.test_integrity()
    raw_input("Press any key to continue")
        