#Remote imports
import pygame
import string

#Local imports
import config
from events import *
from pgu import gui

#--------------------------------------------------------------------
#New GUI Classes
#--------------------------------------------------------------------
class ImageButton(gui.button._button):
    
    def __init__(self, image, image_hover=None, image_down=None, **params):
        gui.button._button.__init__(self,**params)
        self.ev_manager = params['ev_manager']
        self.style.image = image
        if image_hover == None:
            self.style.image_hover = self.style.image
        else:
            self.style.image_hover = image_hover
        if image_down == None:
            self.style.image_down = self.style.image
        else:
            self.style.image_down = image_down
        widths = [self.style.image.get_width(), self.style.image_hover.get_width(), 
                  self.style.image_down.get_width()]
        heights = [self.style.image.get_height(), self.style.image_hover.get_height(), 
                  self.style.image_down.get_height()]        
        self.style.width = max(widths)
        self.style.height = max(heights)
        
        self.image_pos = self.style.image.get_rect(center=(self.style.width/2, self.style.height/2))
        self.image_hover_pos = self.style.image_hover.get_rect(center=(self.style.width/2, self.style.height/2))
        self.image_down_pos = self.style.image_down.get_rect(center=(self.style.width/2, self.style.height/2))
        
        self.state = 0

    def paint(self,s):
        if self.pcls == "hover":
            s.blit(self.style.image_hover,self.image_hover_pos)
        elif self.pcls == "down":          
            s.blit(self.style.image_down,(self.image_down_pos)) 
        else:        
            s.blit(self.style.image,self.image_pos) 
            
class ZoomImage(pygame.sprite.Sprite):
    
    def __init__(self, ev_manager, image, zoom_time=config.default_zoom_time, pos=None):
        if pos == None:
            self.centered = True
            self.pos = image.get_rect(center=config.screen_center)
        else:
            self.centered = False
            self.pos = pos
        self.ev_manager = ev_manager
        self.ev_manager.register_listener(self)
        self.image = image
        self.rect = self.image.get_rect()
        self.zoomed_image = image
        self.original_size = self.image.get_size()
        self.image_size = (1, 1)
        self.zoom_time = zoom_time
        self.zoom_h_per_ms = float(self.original_size[0]) / float(self.zoom_time)
        self.zoom_v_per_ms = float(self.original_size[1]) / float(self.zoom_time)
        self.finished_zoom_in = False
        self.finished = False
        self.start_zoom_out = False
        self.visible = False
        self.ev_manager.post(FreezeCards())
        self.lifetime = 0
    
    def notify(self, event):
        if isinstance(event, TickEvent):
            if self.image_size < self.original_size and not self.start_zoom_out:
                self.zoom_in(event.time_passed)
            elif self.lifetime < config.zoom_lifetime:
                self.finished_zoom_in = True
                self.lifetime += event.time_passed
            elif not self.finished:
                self.start_zoom_out = True
                self.zoom_out(event.time_passed)
                
    def zoom_in(self, time_passed):
        nh = self.zoom_h_per_ms*time_passed
        nv = self.zoom_v_per_ms*time_passed
        new_image_size_h = min(self.image_size[0] + nh, self.original_size[0])
        new_image_size_v = min(self.image_size[1] + nv, self.original_size[1])
        self.image_size = (int(new_image_size_h), int(new_image_size_v))
        self.zoomed_image = pygame.transform.scale(self.image, self.image_size)
        if self.centered:
            self.pos = self.zoomed_image.get_rect(center=config.screen_center)
        self.visible = True       
        
    def zoom_out(self, time_passed):
        nh = self.zoom_h_per_ms*time_passed
        nv = self.zoom_v_per_ms*time_passed
        new_image_size_h = max(self.image_size[0] - nh, 0)
        if new_image_size_h <= 0:
            self.finished = True
        new_image_size_v = max(self.image_size[1] - nv, 0)
        self.image_size = (int(new_image_size_h), int(new_image_size_v))
        self.zoomed_image = pygame.transform.scale(self.image, self.image_size)
        if self.centered:
            self.pos = self.zoomed_image.get_rect(center=config.screen_center)            
        
    def render(self, surface):
        if self.visible and not self.finished:
            surface.blit(self.zoomed_image, self.pos)

class FadeOut(pygame.Surface):
    
    def __init__(self, ev_manager, goto, fade_time=config.default_fade_time, color=(255,255,255)):
        
        pygame.Surface.__init__(self, config.screen_size)
        self.visible = False
        self.ev_manager = ev_manager
        self.ev_manager.register_listener(self)
        self.goto = goto
        self.fade_per_ms = float(255)/float(fade_time)
        self.fill(color)
        self.alpha = 0
        
    def notify(self, event):
        if isinstance(event, TickEvent):
            if self.alpha >= 1 and not self.visible:
                self.visible = True
            elif self.alpha >= 255:
                self.finished = True   
                self.ev_manager.post(ChangeRoomRequest(self.goto))     
            else:
                self.alpha += (self.fade_per_ms * event.time_passed)
                self.set_alpha(self.alpha)
            
    def render(self, surface):
        if self.visible:
            surface.blit(self, (0,0))

#--------------------------------------------------------------------
#Title Screen GUI
#--------------------------------------------------------------------
class StartGameButton(gui.Button):
    def __init__(self, **params):
        params['value'] = 'Start Game'
        gui.Button.__init__(self, **params)
        self.connect(gui.CLICK,self.clicked,None)
        self.ev_manager = params['ev_manager']
        
    def clicked(self, value):
        self.ev_manager.post(ChangeRoomRequest(config.GAME_MODE_ROOM))


#--------------------------------------------------------------------
#Game Mode Screen GUI
#--------------------------------------------------------------------
class GameModeRelaxedButton(ImageButton):
    def __init__(self, **params):
        image = pygame.image.load('assets/images/interface/button_relaxed.png').convert()
        image_hover = pygame.image.load('assets/images/interface/button_relaxed_hover.png').convert()
        ImageButton.__init__(self, image, image_hover, **params)
        self.connect(gui.CLICK,self.clicked,None)
        
    def clicked(self, value):
        config.game_mode = config.RELAXED
        self.ev_manager.post(ChangeRoomRequest(config.GAME_ROOM, game_mode=config.RELAXED))
        
class TimeChallengeButton(ImageButton):
    def __init__(self, **params):
        image = pygame.image.load('assets/images/interface/button_time_challenge.png').convert()
        ImageButton.__init__(self, image, **params)
        self.connect(gui.CLICK,self.clicked,None)
        
    def clicked(self, value):
        config.game_mode = config.TIME_CHALLENGE
        self.ev_manager.post(ChangeRoomRequest(config.GAME_ROOM, game_mode=config.TIME_CHALLENGE))    
        
class PlanAheadButton(ImageButton):
    def __init__(self, **params):
        image = pygame.image.load('assets/images/interface/button_plan_ahead.png').convert()
        ImageButton.__init__(self, image, **params)
        self.connect(gui.CLICK,self.clicked,None)
        
    def clicked(self, value):
        config.game_mode = config.PLAN_AHEAD
        self.ev_manager.post(ChangeRoomRequest(config.GAME_ROOM, game_mode=config.PLAN_AHEAD))                    

#--------------------------------------------------------------------
#Game Room GUI
#--------------------------------------------------------------------

class HighScoreDialog(gui.Dialog):   
    
    def __init__(self, value="", **params):
        self.ev_manager = params['ev_manager']
        self.ev_manager.register_listener(self)    
        self.highs_table = config.current_highs
        self.score = params['score']
        self.data = params['data']
        title = gui.Label("New High Score!")
        main = gui.Table()   
        
        self.gui_form = gui.Form()
        
        main.tr()
        main.td(gui.Label("Please enter your name:"))
        main.tr()
        main.td(gui.Input(value=value, name='player_name'))
        main.tr()
        submit_button = gui.Button("Submit")
        main.td(submit_button)
        submit_button.connect(gui.CLICK, self.submit, None)
        submit_button.connect(gui.CLOSE, self.close, None)

        gui.Dialog.__init__(self,title,main)   
        
        self.ev_manager.post(FreezeCards())
        
    def close(self, value):    
        self.ev_manager.post(ChangeRoomRequest(config.TITLE_SCREEN))
        
    def submit(self, value):     
        print "Player name: " + str(self.gui_form['player_name'].value)
        config.current_highs.submit(self.score, self.gui_form['player_name'].value, self.data)
        config.current_highs.save()
        self.ev_manager.post(ChangeRoomRequest(config.HIGH_SCORES_ROOM))
        
    def notify(self, event):
        pass
     
class QuitButton(gui.Button):
    
    def __init__(self, **params):
        params['value'] = 'Quit'
        gui.Button.__init__(self, **params)
        self.connect(gui.CLICK,self.clicked,None)
        self.ev_manager = params['ev_manager']
        
    def clicked(self, value):
        self.ev_manager.post(QuitEvent())
        
class GenerateBoardButton(gui.Button):
    
    def __init__(self, **params):
        params['value'] = 'Create Board'
        gui.Button.__init__(self, **params)
        self.connect(gui.CLICK,self.clicked,None)
        self.ev_manager = params['ev_manager'] 
        self.room = params['room']      
        
    def clicked(self, value):
        self.room.new_board = self.room.deck.deal_new_board(self.room.board)
        self.ev_manager.post(NewBoardComplete(self.room.new_board))
        
class SetBoardSizeContainer(gui.Container):
    
    def __init__(self, value, **params):
        gui.Container.__init__(self, **params)
        self.ev_manager = params['ev_manager']
        self.board = params['board']
        
        self.gui_form = gui.Form()
        
        t = gui.Table(background=(255,255,255), align=-1)
        t.tr()
        t.td(gui.Label("Board Size:"), colspan=2)
        t.tr()
        g = gui.Group(value=value, name='board_size')
        t.td(gui.Radio(g,value=config.BOARD_SMALL))
        t.td(gui.Label("Small"), align=-1)
        t.tr()
        t.td(gui.Radio(g,value=config.BOARD_MED))
        t.td(gui.Label("Med"), align=-1)
        t.tr()
        t.td(gui.Radio(g,value=config.BOARD_LARGE))  
        t.td(gui.Label("Large"), align=-1)
        
        g.connect(gui.CHANGE, self.changed, None)
               
        self.add(t, 0, 0)   
        
    def changed(self, value):
        new_board_size = self.gui_form['board_size'].value
        self.ev_manager.post(ConfigChangeBoardSize(new_board_size))
        if new_board_size == config.BOARD_LARGE:
            new_card_size = config.CARD_SMALL
        elif new_board_size == config.BOARD_MED:
            new_card_size = config.CARD_MED
        elif new_board_size == config.BOARD_SMALL:
            new_card_size = config.CARD_LARGE           
        self.ev_manager.post(ConfigChangeCardSize(new_card_size)) 
        
class FillBoardCheckbox(gui.Table):

    def __init__(self, value, **params): 
        gui.Table.__init__(self, **params)
        self.ev_manager = params['ev_manager']        
        self.gui_form = gui.Form()
        g = gui.Group(value=[config.fill_board], name='fill_board')     
        self.td(gui.Checkbox(g, value=1))
        self.td(gui.Label("Fill board"))    
        g.connect(gui.CHANGE, self.changed, None)
        
    def changed(self, value):
        if self.gui_form['fill_board'].value:
            value = 1
        else:
            value = 0
        self.ev_manager.post(ConfigChangeFillBoard(value))    
        
class UseDarknessCheckbox(gui.Table):

    def __init__(self, value, **params): 
        gui.Table.__init__(self, **params)
        self.ev_manager = params['ev_manager']        
        self.gui_form = gui.Form()
        g = gui.Group(value=[config.use_darkness], name='use_darkness')     
        self.td(gui.Checkbox(g, value=1))
        self.td(gui.Label("Darkness"))
        g.connect(gui.CHANGE, self.changed, None)
        
    def changed(self, value):
        if self.gui_form['use_darkness'].value:
            value = 1
        else:
            value = 0
        self.ev_manager.post(ConfigChangeDarkness(value))              
        
#--------------------------------------------------------------------
#High Scores Room GUI
#--------------------------------------------------------------------    

class HighScoresTable(gui.Table):
    
    def __init__(self, **params): 
        gui.Table.__init__(self, **params)   
        self.tr()
        self.td(gui.Label("Name"))
        self.td(gui.Label("Level"))
        self.td(gui.Label("Time"))
        self.td(gui.Label("Swaps"))      
        
        for hs in config.current_highs:
            self.tr()
            self.td(gui.Label(hs.name))
            self.td(gui.Label(str(hs.score)))  
                
            data_dic = {}
            for item in hs.data.split(","):
                key,value = item.split(":")
                data_dic[key] = value 
            print data_dic

            self.td(gui.Label(data_dic['time']))
            self.td(gui.Label(data_dic['swaps']))   
        
                   