#Remote imports
import pygame

#Local imports
import config
import continents
from game_components import *
from my_gui import *
from pgu import high

class Room(object):

    def __init__(self, screen, ev_manager):
        self.screen = screen
        self.ev_manager = ev_manager
        self.ev_manager.register_listener(self)
        
    def notify(self, event):
        if isinstance(event, TickEvent): 
            pygame.display.set_caption('FPS: ' + str(int(event.fps)))         
            self.render(self.screen) 
            pygame.display.update()

        
    def get_highs_table(self):
        fname = 'high_scores.txt'
        highs_table = None
        config.all_highs = high.Highs(fname)
        if config.game_mode == config.TIME_CHALLENGE:
            if config.difficulty == config.EASY:
                highs_table = config.all_highs['time_challenge_easy']
            if config.difficulty == config.MED_DIF:
                highs_table = config.all_highs['time_challenge_med']
            if config.difficulty == config.HARD:
                highs_table = config.all_highs['time_challenge_hard']
            if config.difficulty == config.SUPER:
                highs_table = config.all_highs['time_challenge_super']                  
        elif config.game_mode == config.PLAN_AHEAD:
            pass     
        return highs_table

class TitleScreen(Room):
    
    def __init__(self, screen, ev_manager):
        Room.__init__(self, screen, ev_manager)
        
        self.background = pygame.image.load('assets/images/interface/background.jpg').convert()      
        #Initialize
        #---------------------------------------
        self.gui_form = gui.Form()
        self.gui_app = gui.App(config.gui_theme)
        self.ev_manager.gui_app = self.gui_app
        c = gui.Container(align=0,valign=0)        
        
        #Quit Button
        #---------------------------------------
        b = StartGameButton(ev_manager=self.ev_manager)
        c.add(b, 0, 0)    
        self.gui_app.init(c)
    
        
    def render(self, surface):
        surface.blit(self.background, (0, 0))
        #GUI
        self.gui_app.paint(surface)            

class GameModeRoom(Room):
    
    def __init__(self, screen, ev_manager):
        Room.__init__(self, screen, ev_manager)

        self.background = pygame.image.load('assets/images/interface/background.jpg').convert()      
        self.create_gui()

    #Create pgu gui elements
    def create_gui(self):
        #Setup
        #---------------------------------------
        self.gui_form = gui.Form()
        self.gui_app = gui.App(config.gui_theme)
        self.ev_manager.gui_app = self.gui_app
        c = gui.Container(align=0,valign=-1)        
        
        #Mode Relaxed Button
        #---------------------------------------
        b = GameModeRelaxedButton(ev_manager=self.ev_manager)
        self.b = b
        print b.rect
        c.add(b, 0, 200)    
        
        #Mode Time Challenge Button
        #---------------------------------------
        b = TimeChallengeButton(ev_manager=self.ev_manager)
        self.b = b
        print b.rect
        c.add(b, 0, 250)    
        
        #Mode Think Ahead Button
        #---------------------------------------
#        b = PlanAheadButton(ev_manager=self.ev_manager)
#        self.b = b
#        print b.rect
#        c.add(b, 0, 300)            
        
        #Initialize
        #---------------------------------------       
        self.gui_app.init(c)          
  
    
    def render(self, surface):
        surface.blit(self.background, (0, 0))
        #GUI
        self.gui_app.paint(surface)    
        
class GameRoom(Room):
    
    def __init__(self, screen, ev_manager):
        Room.__init__(self, screen, ev_manager)
        
        #Game mode
        #---------------------------------------   
        self.new_board_timer = None        
        self.game_mode = config.game_mode
        config.current_highs = self.get_highs_table()
        self.highs_dialog = None
        self.game_over = False
 
        #Images
        #---------------------------------------   
        self.background = pygame.image.load('assets/images/interface/game screen2-1.jpg').convert() 
        self.logo = pygame.image.load('assets/images/interface/logo_small.png').convert_alpha() 
        self.game_over_text = pygame.image.load('assets/images/interface/text_game_over.png').convert_alpha()
        self.trip_complete_text = pygame.image.load('assets/images/interface/text_trip_complete.png').convert_alpha()
        self.zoom_game_over = None
        self.zoom_trip_complete = None
        self.fade_out = None
        
        #Text
        #---------------------------------------   
        self.font = pygame.font.Font(config.font_sans, config.interface_font_size)
        
        #Create game components   
        #---------------------------------------      
        self.continent = self.set_continent(config.continent)
        self.board = Board(config.board_size, self.ev_manager)
        self.deck = Deck(self.ev_manager, self.continent)
        self.map = Map(self.continent)
        self.longest_trip = 0
  
        #Set pos of game components
        #---------------------------------------   
        board_pos = (SCREEN_MARGIN[0], 109)
        self.board.set_pos(board_pos)
        map_pos = (config.screen_size[0] - self.map.size[0] - SCREEN_MARGIN[0], 57);
        self.map.set_pos(map_pos)        
        
        #Trackers  
        #---------------------------------------              
        self.game_clock = Chrono(self.ev_manager)
        self.swap_counter = 0
        self.level = 0    
        
        #Create gui
        #---------------------------------------   
        self.create_gui()
        
        #Create initial board
        #---------------------------------------   
        self.new_board = self.deck.deal_new_board(self.board)
        self.ev_manager.post(NewBoardComplete(self.new_board)) 
        
    def set_continent(self, continent_const):
        #Set continent based on const
        if continent_const == config.EUROPE:
            return continents.Europe()
        if continent_const == config.AFRICA:
            return continents.Africa()        
        else:
            raise Exception('Continent constant not recognized')     
    
    #Create pgu gui elements
    def create_gui(self):
        
        #Setup
        #---------------------------------------
        self.gui_form = gui.Form()
        self.gui_app = gui.App(config.gui_theme)
        self.ev_manager.gui_app = self.gui_app
        c = gui.Container(align=-1,valign=-1)        
        
        #Timer Progress bar
        #---------------------------------------
        self.timer_bar = None
        self.time_increase = None
        self.minutes_left = None
        self.seconds_left = None
        self.timer_text = None
        if self.game_mode == config.TIME_CHALLENGE:
            self.time_increase = config.time_challenge_start_time
            self.timer_bar = gui.ProgressBar(config.time_challenge_start_time,0,config.max_time_bank,width=306)
            c.add(self.timer_bar, 172, 57)
            
        #Connections Progress bar
        #---------------------------------------
        self.connections_bar = None
        self.connections_bar = gui.ProgressBar(0,0,config.longest_trip_needed,width=306)
        c.add(self.connections_bar, 172, 83)            
        
        #Quit Button
        #---------------------------------------
        b = QuitButton(ev_manager=self.ev_manager)
        c.add(b, 950, 20)
        
        #Generate Board Button
        #---------------------------------------
        b = GenerateBoardButton(ev_manager=self.ev_manager, room=self)
        c.add(b, 500, 20)        
        
        #Board Size?
        #---------------------------------------
        bs = SetBoardSizeContainer(config.BOARD_LARGE, ev_manager=self.ev_manager, board=self.board)
        c.add(bs, 640, 20)         
        
        #Fill Board?
        #---------------------------------------  
        t = FillBoardCheckbox(config.fill_board, ev_manager=self.ev_manager)
        c.add(t, 740, 20)
        
        #Darkness?
        #---------------------------------------  
        t = UseDarknessCheckbox(config.use_darkness, ev_manager=self.ev_manager) 
        c.add(t, 840, 20)        

        #Initialize
        #---------------------------------------    
        self.gui_app.init(c)
        
    def advance_level(self):
        self.level += 1
        print 'Advancing to next level'        
        print 'New level: ' + str(self.level)
        if self.timer_bar:      
            print 'Time increase: ' + str(self.time_increase)
            self.timer_bar.value += self.time_increase
            self.time_increase = max(config.min_advance_time, int(self.time_increase * 0.9))            
        self.board = self.new_board
        self.new_board = None
        self.zoom_trip_complete = None
        self.game_clock.unpause()
 
    def notify(self, event):
        #Tick event
        if isinstance(event, TickEvent): 
            pygame.display.set_caption('FPS: ' + str(int(event.fps)))         
            self.render(self.screen) 
            pygame.display.update()
            
            #Wait to deal new board when advancing levels
            if self.zoom_trip_complete and self.zoom_trip_complete.finished:
                self.zoom_trip_complete = None
                self.ev_manager.post(UnfreezeCards())                    
                self.new_board = self.deck.deal_new_board(self.board)
                self.ev_manager.post(NewBoardComplete(self.new_board))
                     
            #New high score?
            if self.zoom_game_over and self.zoom_game_over.finished and not self.highs_dialog:
                if config.current_highs.check(self.level) != None:
                    self.zoom_game_over.visible = False
                    data = 'time:' + str(self.game_clock.time) + ',swaps:' + str(self.swap_counter)
                    self.highs_dialog = HighScoreDialog(score=self.level, data=data, ev_manager=self.ev_manager)
                    self.highs_dialog.open()
                elif not self.fade_out:
                    self.fade_out = FadeOut(self.ev_manager, config.TITLE_SCREEN)
                    
        #Second event
        elif isinstance(event, SecondEvent):
            if self.timer_bar:
                if not self.game_clock.paused:      
                    self.timer_bar.value -= 1 
                if self.timer_bar.value <= 0 and not self.game_over:
                    self.ev_manager.post(GameOver())
                self.minutes_left = self.timer_bar.value / 60
                self.seconds_left = self.timer_bar.value % 60
                if self.seconds_left < 10:
                    leading_zero = '0'
                else:
                    leading_zero = ''
                self.timer_text = ''.join(['Time Left: ', str(self.minutes_left), ':', leading_zero, str(self.seconds_left)])
                    
        #Game over
        elif isinstance(event, GameOver):
            self.game_over = True
            self.zoom_game_over = ZoomImage(self.ev_manager, self.game_over_text)
                
        #Trip complete event
        elif isinstance(event, TripComplete):         
            print 'You did it!'
            self.game_clock.pause()
            self.zoom_trip_complete = ZoomImage(self.ev_manager, self.trip_complete_text)
            self.new_board_timer = Timer(self.ev_manager, 2)
            self.ev_manager.post(FreezeCards())
            print 'Room posted newboardcomplete'  
            
        #Board Refresh Complete
        elif isinstance(event, BoardRefreshComplete):
            if event.board == self.board:
                print 'Longest trip needed: ' + str(config.longest_trip_needed)
                print 'Your longest trip: ' + str(self.board.longest_trip)
                if self.board.longest_trip >= config.longest_trip_needed:
                    self.ev_manager.post(TripComplete())            
            elif event.board == self.new_board:
                self.advance_level()
            self.connections_bar.value = self.board.longest_trip
            self.connection_text = ' '.join(['Connections:', str(self.board.longest_trip), '/', str(config.longest_trip_needed)])
                
        #CardSwapComplete
        elif isinstance(event, CardSwapComplete):
            self.swap_counter += 1     
            
        elif isinstance(event, ConfigChangeBoardSize): 
            config.board_size = event.new_size
        elif isinstance(event, ConfigChangeCardSize): 
            config.card_size = event.new_size       
        elif isinstance(event, ConfigChangeFillBoard): 
            config.fill_board = event.new_value          
        elif isinstance(event, ConfigChangeDarkness): 
            config.use_darkness = event.new_value                        
            
    def render(self, surface):
        #Background
        surface.blit(self.background, (0, 0))
        
        #Map
        self.map.render(surface)  
        
        #Board      
        self.board.render(surface)
        
        #Logo
        surface.blit(self.logo, (10,10))
        
        #Text
        connection_text = self.font.render(self.connection_text, True, BLACK)
        surface.blit(connection_text, (25, 84))
        if self.timer_text:
            timer_text = self.font.render(self.timer_text, True, BLACK)
            surface.blit(timer_text, (25, 64))        
        
        #GUI
        self.gui_app.paint(surface)   
        if self.zoom_trip_complete:
            self.zoom_trip_complete.render(surface)     
        if self.zoom_game_over:
            self.zoom_game_over.render(surface)
        if self.fade_out:
            self.fade_out.render(surface)
            
class HighScoresRoom(Room):
    
    def __init__(self, screen, ev_manager):
        Room.__init__(self, screen, ev_manager)
        
        self.background = pygame.image.load('assets/images/interface/background.jpg').convert()      
        #Initialize
        #---------------------------------------
        self.gui_app = gui.App(config.gui_theme)
        self.ev_manager.gui_app = self.gui_app
        c = gui.Container(align=0,valign=0)        
        
        #High Scores Table
        #---------------------------------------
        hst = HighScoresTable()
        c.add(hst, 0, 0)    
        self.gui_app.init(c)
    
        
    def render(self, surface):
        surface.blit(self.background, (0, 0))
        #GUI
        self.gui_app.paint(surface)      
    